Candidates

Struct Candidates 

pub struct Candidates<'a> { /* private fields */ }
The set of mutations that can be applied to a value.

This type is used by mutators to register the mutations that they can perform on a value. It is passed to the Mutate::mutate trait method, and provides a way to register candidate mutations, as well as to check if shrinking is enabled.

Implementations

Source

impl<'a> Candidates<'a>

pub fn mutation( &mut self, f: impl FnMut(&mut Context) -> Result<()>, ) -> Result<()>

Register a candidate mutation that can be applied to a value.

This method is called by Mutate::mutate implementations to register the potential mutations that they can perform on a value.

f should be a closure that performs the mutation on the value that was passed to Mutate::mutate, updating the value and the mutator itself as necessary.

See the Mutate::mutate trait method documentation for more information on this method’s use.

pub fn shrink(&self) -> bool

Whether only shrinking mutations should be registered in this mutation set or not.

When this method returns true, then you should not register any mutation which can grow the value being mutated.