#[non_exhaustive]pub enum ErrorKind {
Exhausted,
InvalidRange,
Other(ErrorMessage),
}Expand description
The kind of an error that can occur when using the mutatis crate.
This enum is not exhaustive, and new variants may be added in the future. When matching on this enum, a catch-all arm should be used to handle any new variants that are added.
Examples
use mutatis::error::{Error, ErrorKind};
let error: Error = {
// ...
};
match error.kind() {
ErrorKind::Exhausted => println!("exhausted!"),
ErrorKind::Other(msg) => println!("other! {msg}"),
unknown => println!("unknown! {unknown:?}"),
}Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Exhausted
The mutator is exhausted.
InvalidRange
The mutator was given an invalid range.
Other(ErrorMessage)
Some other error occurred.